local EEAB = _G.EEAB

function EEAB.CheckForEquip(this)
	local info = {}
	info.typ = "equip"
	info.ID = EEAB.Data(this, "ID")

	if not info.ID then
		return
	end

	info.texture = GetInventoryItemTexture("player", info.ID)
	info.name = EEAB.Data(this, "name")

	if not info.texture then
		_, info.texture = GetInventorySlotInfo(info.name)
	end

	EEAB.Data(this, info)
end

function EEAB.CheckForSkin(this)
	local info = {}
	info.typ = "CoolSuit"
	info.slot = EEAB.Data(this, "slot")
	_, info.texture = CoolSuit_GetItemInfo(info.slot, 3)

	if not info.texture then
		for i = 1, 9 do
			_, info.texture = CoolSuit_GetItemInfo(info.slot, i)

			if info.texture then
				break
			end
		end
	end

	if not info.texture then
		info.texture = CoolSuit_GetPageTexture(info.slot)
	end

	EEAB.Data(this, info)
end

function EEAB.CheckForMacro(this)
	local info = {}
	info.typ = "macro"
	info.ID = EEAB.Data(this, "ID")
	info.count, info.name = GetMacroInfo(info.ID)

	if info.count then
		info.texture = GetMacroIconInfo(info.count)
		EEAB.Data(this, info)
	else
		EEAB.Data(this, {})
	end

	EEAB.AddUpdate(this)
end

function EEAB.SuitSkillCooldown(this)
	local delay = 0.05
	local name = EEAB.Data(this, "name")

	if name == TEXT("Sys495478_name") then
		EEAB.Spellcache[name] = {
			cooldown = GetActionCooldown(1),
			remaining = GetActionCooldown(1),
			frame = this:GetName(),
			delay = 18000,
		}
	else
		EEAB.Spellcache[name] = {
			cooldown = GetActionCooldown(1) - delay,
			remaining = GetActionCooldown(1) - 0.00000000001 - delay,
			frame = this:GetName(),
			delay = delay,
		}
	end
end

function EEAB.GetActionInfo(this)
	local texture, name, count, locked, wore, continued, invalid

	texture		=	EEAB.Data(this, "texture") or ""
	name		=	EEAB.Data(this, "name") or ""

	if EEAB_Data.Options.slim then
		count 	=	0
	else
		count	=	EEAB.Count(this, EEAB.Data(this, "typ"))
	end

	locked		=	false --???
	wore		=	EEAB.Data(this, "typ") == "equip"

	continued	=	EEAB.CheckBuffs(this)

	if EEAB_Data.Options.slim then
		invalid =	false
	else
		invalid	=	EEAB.CheckInvalid(this)
	end

	local nio = {"bag", "itemshop", "skill", "SuitSkill", "CoolSuit", "equip"}

	for _, s in ipairs(nio) do
		if s == EEAB.Data(this, "typ") then
			name = ""
		end
	end

	return texture, name, count, locked, wore, continued, invalid
end

function EEAB.CheckInvalid(this)
	local invalid = false

	if EEAB.Data(this, "typ") == "bag" then
		_, _, _, _, invalid = GetBagItemInfo(EEAB.Data(this, "slot"))

	elseif EEAB.Data(this, "typ") == "SuitSkill" then
		local name = EEAB.Data(this, "name")
		invalid = true

		for i = 0, 2 do
			if name == SkillPlateUpdate(i) then
				return false
			end
		end

	elseif EEAB.Data(this, "typ") == "equip" then
		if EEAB.Data(this, "ID") then
			invalid = GetInventoryItemInvalid("player", EEAB.Data(this, "ID"))
		end
	end

	return invalid
end

function EEAB.CheckBuffs(this, returntime) --## for itemshop add debuffscan
	if not EEAB_Data.Options.buffs and not returntime then
		return false
	end

	local name = EEAB.Data(this, "name")

	if EEAB.Data(this, "typ") == "CoolSuit" then
		if EEAB.Data(this, "slot") == CoolSuit_GetPageID() then
			return true
		end
	else
		if EEAB.Data(this, "typ") == "title" then
			name = EEAB.GetTitleSkill(EEAB.Data(this, "slot"))

			if not name or name == "" then
				name = EEAB.Data(this, "name")
			end
		end

		local icon = EEAB.Data(this, "texture")

		local bool = EEAB.CheckSpecialBuffs(name, returntime)

		if bool == nil then
			if name and name ~= "" then
				local i, name2, icon2 = 0, ""

				while name2 ~= nil do
					i = i + 1

					name2, icon2 = UnitBuff("player", i)

					if name2 and name == name2 then
						if not returntime then
							return true
						else
							return i
						end

					elseif icon2 and icon2 == icon then
						if not returntime then
							return true
						else
							return i
						end
					end
				end
			end
		else
			return bool
		end
	end

	return false
end

function EEAB.CheckSpecialBuffs(name, returntime) --## check for spezial cases: different items use the same icon. Improve Database
	local bool

	local checker = {
		--	item,	buff,		note in german
		{207206,	495644},	--Trank: Scharlachrote Liebe	Scharlachrote Liebe
		{201141,	501389},	--Erlösung des Phönix			Phoenix's Redemption
		{492423,	492423},	--Heiliger Phönix				Heiliger Phönix

		{201610,	501592},	--Fertigkeitstrank, 			Fertigkeitstrank
		{207202,	495640},	--Trank: Klarer Gedanke			Klarer Gedanke

		{202264,	501388},	--Erfahrungstrank, 				Erfahrungstrank
		{200194,	490763},	--Lebensquelle					Lebensquelle

		{207203,	506687},	--Trank: Glückliches Ziel		Glückspuderstaub

		{204532,	493069},	--Magisches Saxophon (Prototyp)	Saxophon spielen
		{204542,	493069},	--Magisches Saxophon (3 Tage)	Saxophon spielen
		{204568,	493069},	--Magisches Saxophon (7 Tage)	Saxophon spielen
		{206417,	493069},	--Magisches Saxophon (30 Tage)	Saxophon spielen

		{204533,	493068},	--Magisches Gitarre (Prototyp)	Gitarre spielen
		{204543,	493068},	--Magisches Gitarre (3 Tage)	Gitarre spielen
		{204569,	493068},	--Magisches Gitarre (7 Tage)	Gitarre spielen
		{206418,	493068},	--Magisches Gitarre (30 Tage)	Gitarre spielen

		{204534,	493067},	--Magisches Laute (Prototyp)	Laute spielen
		{204462,	493067},	--Magisches Laute (3 Tage)		Laute spielen
		{204570,	493067},	--Magisches Laute (7 Tage)		Laute spielen
		{206419,	493067},	--Magisches Laute (30 Tage)		Laute spielen

		{204535,	493071},	--Magisches Tamburin (Prototyp)	Tamburin schlagen
		{204463,	493071},	--Magisches Tamburin (3 Tage)	Tamburin schlagen
		{204571,	493071},	--Magisches Tamburin (7 Tage)	Tamburin schlagen
		{206420,	493071},	--Magisches Tamburin (30 Tage)	Tamburin schlagen

		{204536,	493070},	--Magische Rassel (Prototyp)	Rassel schütteln
		{204464,	493070},	--Magische Rassel (3 Tage)		Rassel schütteln
		{204572,	493070},	--Magische Rassel (7 Tage)		Rassel schütteln
		{206421,	493070},	--Magische Rassel (30 Tage)		Rassel schütteln
	}

	for _, value in ipairs(checker) do
		local item, result = value[1], value[2]

		if name == TEXT("Sys"..item.."_name") then
			local i, name2 = 0, ""

			while name2 ~= nil do
				i = i + 1

				name2 = UnitBuff("player", i)

				if name2 and TEXT("Sys"..result.."_name") == name2 then
					if not returntime then
						return true
					else
						return i
					end
				end
			end

			return false
		end
	end

	return bool
end

function EEAB.CheckSpecialDebuffs(name) --## check if working.. database
	local duration, remaining

	local checker = {
		--	item,	buff,		note in german
		{	201141,	501617},	--Erlösung des Phönix			Phönix' Unterbindung
	}

	for _, value in ipairs(checker) do
		local item, result = value[1], value[2]

		if name == TEXT("Sys"..item.."_name") then
			local i, name2 = 0, ""

			while name2 ~= nil do
				i = i + 1

				name2 = UnitDebuff("player", i)

				if name2 and TEXT("Sys"..result.."_name") == name2 then
					remaining = UnitDebuffLeftTime("player", i)

					if remaining then
						if not EEAB[result] or (EEAB[result] and EEAB[result] < remaining) then
							EEAB[result] = remaining
						end
					else
						EEAB.Log("no remaining time")
					end

					duration = EEAB[result]

					EEAB.Log(duration) --## correct?
					EEAB.Log(remaining) --## correct?
				end
			end
		end
	end

	return duration, remaining
end

function EEAB.HasAction(this)
	if not EEAB.Data(this, "texture") or EEAB.Data(this, "texture") == "" then
		return false
	else
		return true
	end
end

function EEAB.GetActionCooldown(this)
	local duration, remaining

	local info = {
		typ		= EEAB.Data(this, "typ"),
		name	= EEAB.Data(this, "name"),
		ID		= EEAB.Data(this, "ID"),
		slot	= EEAB.Data(this, "slot"),
		count	= EEAB.Data(this, "count"),
	}

	this = getglobal(this:GetName().."_Cooldown")

	if this.starting and this.starting == 1 then
		duration = this.duration
		remaining = this.remaining
	else
		local check

		if not EEAB_Data.Options.slim and EEAB.LastCooldown then
			check =  EEAB.LastCooldown[info.name]
		end

		if check then
			duration = check.duration
			remaining = check.remaining

		elseif info.typ == "bag" and info.count ~= 0 then
			duration, remaining = GetBagItemCooldown(GetBagItemInfo(info.slot))

		elseif info.typ == "itemshop" and info.count ~= 0 then
			duration, remaining = EEAB.CheckSpecialDebuffs(info.name)

		elseif info.typ == "skill" then
			duration, remaining = GetSkillCooldown(info.slot, info.ID)

		elseif info.typ == "equip" then
			duration, remaining = GetEquipItemCooldown(info.ID)

		elseif info.typ == "macro" or info.typ == "SuitSkill" then
			return
		end
	end

	return duration or 0, remaining or 0
end

function EEAB.GetMacroCooldown(this) --## alpha
	local duration, content, bool = 0, nil, false

	_, _, content = GetMacroInfo(EEAB.Data(this, "ID"))

	if string.find(content, "/wait %d+") then
		local search = string.gfind(content, "/wait %d+")

		while not bool do
			local num = search()

			if num then
				duration = duration + string.sub(num, 7)
			else
				bool = true
			end
		end
	end

	EEAB.CooldownFrame_SetTime(getglobal(this:GetName().."_Cooldown"), duration, duration - 0.00000000001)
end

function EEAB.CooldownFrame_SetTime(this, duration, remaining)
	local name = EEAB.Data(this:GetParent(), "name")

	if name and EEAB.LastCooldown and duration >= 1 then
		EEAB.LastCooldown[name] = {
			["duration"] = duration,
			["remaining"] = remaining,
		}
	end

	CooldownFrame_SetTime(this, duration, remaining)
end

function EEAB.floor(val, digits)
	digits = digits or 0

	val = math.floor(val * 10^digits + 0.5) / 10^digits

	if digits > 0 and val * 10^(digits - 1) == math.floor(val * 10^(digits - 1)) then
		val = tostring(val).."."..string.rep("0", digits)
	end

	return tostring(val)
end

function EEAB.GetTitleSkill(ID)
	ExtendedActionBar_GameTooltip:SetTitle(ID)

	local skill = ExtendedActionBar_GameTooltipTextLeft2:GetText() or ""

	ExtendedActionBar_GameTooltipTextLeft2:SetText("")
	ExtendedActionBar_GameTooltip:Hide()

	local beginning = string.find(TEXT("TITLE_TIP_SKILL_FORMAT"), "<skill>")
	local ending = string.find(skill, "|r")

	if beginning and ending then
		skill = string.sub(skill, beginning, ending - 1)
	end

	return skill
end

function EEAB.TrafficColor(part, total)
	local part = tonumber(part)
	local red, green, blue = 0, 0, 0

	green = part / total * 510

	red = 510 - green

	if green > 255 then
		green = 255
	end

	if red < 0 then
		red = 0
	end

	return red / 255, green / 255, blue / 255
end

function EEAB.SaveBuffTimeOnFrame(this)
	local i = EEAB.CheckBuffs(this:GetParent(), true)

	if tonumber(i) and UnitBuffLeftTime("player", i) then
		if not this.remaining then
			this:Show()
			this.duration = UnitBuffLeftTime("player", i)
			this.remaining = this.duration
		else
			this.remaining = UnitBuffLeftTime("player", i)
		end
	end
end
