local EEAB = _G.EEAB

function EEAB.Bar_GetNum()
	return 6
end

function EEAB.Bar_OnLoad(this)
	EEAB.Bar_Update(this)
end

function EEAB.Bar_OnEvent(this, event)
end

EEAB.Bar_OnUpdate_counter = 0

function EEAB.Bar_OnUpdate(this, elapsedTime)
	if not EEAB_Data.Options.slim or EEAB.Bar_OnUpdate_counter >= 0.1 then
		if EEAB.Bar_OnUpdate_counter then
			elapsedTime = EEAB.Bar_OnUpdate_counter
		end

		if this.Update then
			if this.Update <= 0 then
				this.Update = nil
				EEAB.Bar_Update(this)
			else
				this.Update = this.Update - elapsedTime
			end
		else
			this.Update = 60
		end

		EEAB.Bar_OnUpdate_counter = 0
	else
		EEAB.Bar_OnUpdate_counter = EEAB.Bar_OnUpdate_counter + elapsedTime
	end
end

function EEAB.Bar_UpdateAll(bool)
	for i = 1, EEAB.Bar_GetNum() do
		this = getglobal("ExtendedActionBar_"..i)

		if this then
			if bool then
				this.hash = nil

				this:Hide()
			end

			this.Update = nil

			EEAB.Bar_Update(this)
		end
	end
end

function EEAB.GoToUIParent()
	local x, y = GetCursorPos()
	local scale = UIParent:GetRealScale()

	x = x / scale
	y = y / scale

	return x, y
end

function EEAB.CheckButtonAnchor(button, this)
	local _, _, parent = button:GetAnchor()

	if not button:IsVisible() then
		return false

	elseif this:GetName() == button:GetName() then
		return false
	end

	for i = 1, 100 do
		_, _, button, x2, y2 = button:GetAnchor()

		if not button then
			break

		elseif button:GetName() == this:GetName() then
			return false
		end
	end

	return true
end

function EEAB.SaveBarPos(this)
	this.hash = nil

	local x, y = this:GetPos()
	local realScale = UIParent:GetRealScale()
	x, y = x / realScale, y / realScale

	local name = this:GetName()
	local scale = EEAB.GetBarScale(this)
	local nextframe = {}

	EEAB_Data["Options"][EEAB.GetClass()][name][4] = EEAB.Rel(x, true)
	EEAB_Data["Options"][EEAB.GetClass()][name][5] = EEAB.Rel(y, false)

	if EEAB.GetBarFree(this) then
		x, y = this:GetPos()

		for i = 1, EEAB.Bar_GetNum() do
			local bar = getglobal("ExtendedActionBar_"..i)

			for i = 1, 20 do
				local button = getglobal(bar:GetName().."_Button"..i)

				if EEAB.CheckButtonAnchor(button, this) then
					local x2, y2 = button:GetPos()
					local distance = math.sqrt((x - x2)^2 + (y - y2)^2)

					if distance <= scale + 4 then
						if not nextframe.name or nextframe.distance > distance then
							nextframe = {
								name	= button:GetName(),
								distance= distance,
								x		= x2,
								y		= y2,
							}
						end
					end
				end
			end
		end

		if nextframe.name then
			local relativTo, relativPoint, relativX, relativY

			if math.abs(y - nextframe.y) > math.abs(x - nextframe.x) then
				if y > nextframe.y then
					relativTo = "TOP"
					relativPoint = "BOTTOM"

					relativY = 2
				else
					relativTo = "BOTTOM"
					relativPoint = "TOP"

					relativY = -2
				end

				relativX = 0
			else
				if x > nextframe.x then
					relativTo = "LEFT"
					relativPoint = "RIGHT"

					relativX = 2
				else
					relativTo = "RIGHT"
					relativPoint = "LEFT"

					relativX = -2
				end

				relativY = 0
			end

			EEAB_Data["Options"][EEAB.GetClass()][name] = {
				[1] = relativTo,
				[2] = relativPoint,
				[3] = nextframe.name,
				[4] = EEAB.Rel(relativX, true),
				[5] = EEAB.Rel(relativY, false),
			}
		else
			x, y = EEAB.GoToUIParent()

			local x, y, anchor = EEAB.ChooseAnchor(x, y)

			EEAB_Data["Options"][EEAB.GetClass()][name] = {
				[1] = "TOPLEFT",
				[2] = anchor,
				[3] = "UIParent",
				[4] = EEAB.Rel(x, true),
				[5] = EEAB.Rel(y, false),
			}
		end
	else
		local x, y, anchor = EEAB.ChooseAnchor(x, y)

		EEAB_Data["Options"][EEAB.GetClass()][name][2] = anchor
		EEAB_Data["Options"][EEAB.GetClass()][name][4] = EEAB.Rel(x, true)
		EEAB_Data["Options"][EEAB.GetClass()][name][5] = EEAB.Rel(y, false)
	end

	EEAB.Bar_Update(this:GetParent())
end

function EEAB.Bar_Update(this)
	if EEAB_Data and EEAB_Data.Options and EEAB_Data["Options"][EEAB.GetClass()] then
		local info = {
			visible	= EEAB_Data["Options"][EEAB.GetClass()]["show"..this:GetID()],
			count	= EEAB_Data["Options"][EEAB.GetClass()]["num"..this:GetID()],
			row		= EEAB_Data["Options"][EEAB.GetClass()]["size"..this:GetID()],
		}

		if not ImplementActionBarFrame:IsVisible() and info.visible then
			this:Show()
		else
			this:Hide()

			return
		end

		local scale = EEAB.GetBarScale(this)
		local hash = tostring(EEAB.GetBarFree(this))..EEAB.GetClass()..tostring(EEAB_Data.Options.slim)..scale..tostring(count)..tostring(row)

		if not this.hash or this.hash ~= hash then
			this.hash = hash

			EEAB.GenerateAnchors(this, info, scale)

			for i = 1, 20 do
				local button = getglobal(this:GetName().."_Button"..i)
				local anchor = getglobal(button:GetName().."_Anchor")

				local set = EEAB_Data["Options"][EEAB.GetClass()][button:GetName()] or {}

				if set[1] and set[2] and set[3] and set[4] and set[5] then
					button:ClearAllAnchors()
					button:SetAnchor(set[1], set[2], getglobal(set[3]), EEAB.Abs(set[4], true), EEAB.Abs(set[5], false))
				end

				anchor:Show()

				if not EEAB_Data.Options.triangel or (not EEAB_Data["Options"][EEAB.GetClass()][button:GetName()]["anchor"] and not EEAB.GetBarFree(this)) then
					anchor:Hide()
				end

				EEAB.AddUpdate(button)
			end
		end
		
		for i = 1, 20 do
			EEAB.AddUpdate(getglobal(this:GetName().."_Button"..i))
		end
	end
end

function EEAB.GenerateAnchors(this, info, scale)
	local column, row, set = 1, 1

	for i = 1, 20 do
		local button = getglobal(this:GetName().."_Button"..i)

		set = {}

		if EEAB.GetBarFree(this) and EEAB_Data["Options"][EEAB.GetClass()][button:GetName()] then
			set = EEAB_Data["Options"][EEAB.GetClass()][button:GetName()]
			set.anchor = true
		else
			if i > 1 and not set.anchor then
				if row < info.row then
					row = row + 1
					set = {
						"LEFT",
						"RIGHT",
						this:GetName().."_Button"..(i - 1),
						EEAB.Rel(2, true),
						EEAB.Rel(0, false),
					}

					set.anchor = false
				else
					row = 1
					set = {
						"TOP",
						"BOTTOM",
						this:GetName().."_Button"..info.row * (column - 1) + 1,
						EEAB.Rel(0, true),
						EEAB.Rel(2, false),
					}

					column = column + 1
					set.anchor = false
				end
			else
				if not EEAB_Data["Options"][EEAB.GetClass()][button:GetName()] or EEAB_Data["Options"][EEAB.GetClass()][button:GetName()][3] ~= "UIParent" then
					set = {
						"TOPLEFT",
						"TOPLEFT",
						"UIParent",
						50,
						50,
					}
				else
					set = EEAB_Data["Options"][EEAB.GetClass()][button:GetName()]
				end

				set.anchor = true
			end

			if EEAB.GetBarFree(this) then
				EEAB_Data["Options"][EEAB.GetClass()][button:GetName()] = set

				EEAB.GenerateAnchors(this, info, scale)
			end
		end

		if i <= info.count then
			button.visible = true
		else
			button.visible = false
		end

		EEAB.ScaleButton(button, scale)

		EEAB_Data["Options"][EEAB.GetClass()][button:GetName()] = set
	end

	EEAB.ChangeBarSize(this, info, scale)
end

function EEAB.ChangeBarSize(this, info, scale)
	if not EEAB.GetBarFree(this) then
		local column, row = 1, 1

		for i = 1, 20 do
			if i > 1 then
				if row < info.row then
					row = row + 1
				else
					row = 1
					column = column + 1
				end
			end
		end

		this:SetSize(scale * row + (row - 1) * 2 + 2, scale * column + (column - 1) * 2 + 2)
	end

	if EEAB_Data.Options.minimap then
		ExtendedActionBar_MinimapButton:Show()
	else
		ExtendedActionBar_MinimapButton:Hide()
	end
end
