local EEAB = _G.EEAB

EEAB.Focus_counter = 0
EEAB.Set = 60

function EEAB.Focus(this, elapsedTime)
	if EEAB_Data and EEAB_Data.Options and not EEAB_Data.Options.slim or EEAB.Focus_counter >= 0.1 then
		elapsedTime = EEAB.Focus_counter

		if (KeyDefineFrame and KeyDefineFrame:IsVisible()) or
		(AskNumberFrame and AskNumberFrame:IsVisible()) or
		(barF2BindingButton and barF2BindingButton:IsVisible()) then

			this:ClearFocus()

		elseif GetKeyboardFocus() == nil then
			this:SetFocus()
		end

		if EEAB.Set then
			EEAB.Set = EEAB.Set - elapsedTime

			if EEAB.Set <= 0 then
				EEAB.Set = nil
				this:SetFocus()
			end
		end
	else
		EEAB.Focus_counter = EEAB.Focus_counter + elapsedTime
	end
end

function EEAB.Key(key, up)
	local origKey = key

	if key then
		if key ~= "ALT" and key ~= "SHIFT" and key ~= "CTRL" then
			if IsAltKeyDown() then
				key = "alt+"..key

			elseif IsShiftKeyDown() then
				key = "shift+"..key

			elseif IsCtrlKeyDown() then
				key = "ctrl+"..key
			end
		end

		if EEAB.LastKeyStat and EEAB.LastKeyStat ~= key..tostring(up) then
			EEAB.ChooseKey(key, up)

			if key ~= origKey then
				EEAB.CheckForNoCombi(key, origKey, up)
			end
		end

		EEAB.LastKeyStat = key..tostring(up)
	end
end

function EEAB.CheckForNoCombi(key, origKey, up)
	local used = false

	if EEAB_DefaultBindings[key] then
		used = true
	else
		if EEAB_Data["Bindings"][key] then
			used = true
		end
	end

	if not used then
		EEAB.ChooseKey(origKey, up)
	end
end

function EEAB.ChooseKey(key, up)
	if not ExtendedActionBar_KeyCatcher:IsVisible() then
		if not EEAB_DefaultBindings then
			EEAB.GenerateDefaultBindings()
		end

		if EEAB_DefaultBindings[key] then
			EEAB.DoSystemBinding(EEAB_DefaultBindings[key], up)
		else
			local indent = EEAB_Data["Bindings"][key]

			if indent and not up then
				local bar = tonumber(indent[1])

				if not ExtendedActionBar_Optionen:IsVisible() then
					EEAB.Button_DoItem(getglobal("ExtendedActionBar_"..bar.."_Button"..tonumber(indent[2])))
				else
					EEAB.io(27)
				end
			end
		end
	else
		if key ~= "ESCAPE" and key ~= "ALT" and key ~= "SHIFT" and key ~= "CTRL" and key ~= "ENTER" and key ~= "SPACE" then
			 EEAB.HandleHotkeyButton(key)
		end
	end
end

function EEAB.GenerateDefaultBindings()
	EEAB_DefaultBindings = {}

	for i = 1, GetNumBindings() do
		local header, name, key1, key2 = EEAB.GetBindings(i)

		if key1 then
			EEAB_DefaultBindings[key1] = name
		end

		if key2 then
			EEAB_DefaultBindings[key2] = name
		end
	end
end

function EEAB.GetBindings(action)
	local category, orgAction, key1, key2, bar = "", ""

	if GetNumBindings() < action then
		action = action - GetNumBindings()

		for i = 1, EEAB.Bar_GetNum() do
			if action <= i * 20 then
				bar = i

				break
			end
		end

		local button = action - (bar - 1) * 20

		key1, key2 = EEAB.GetBindingKey("EXTENDEDACTIONBAR"..EEAB.null(bar).."BUTTON"..EEAB.null(button))
	else
		category, orgAction, key1, key2 = GetBindings(action)
	end

	return category, orgAction, key1, key2
end

function EEAB.GetBindingKey(action)
	if string.find(action, "EXTENDEDACTIONBAR") then
		if EEAB_Data["Bindings"][tonumber(string.sub(action, 18, 21))] then
			return EEAB_Data["Bindings"][tonumber(string.sub(action, 18, 21))][tonumber(string.sub(action, 28))]

		else
			return nil
		end
	else
		return GetBindingKey(action)
	end
end

function EEAB.SetBindingKey(action, key1, key2)
	if string.find(action, "EXTENDEDACTIONBAR") then
		local bar, button = tonumber(string.sub(action, 18, 21)), tonumber(string.sub(action, 28))

		if key1 and key1 ~= "" then
			if EEAB_Data["Bindings"][key1] then
				local pos = EEAB_Data["Bindings"][key1]

				EEAB_Data["Bindings"][key1] = nil
				EEAB_Data["Bindings"][pos[1]][pos[2]] = nil
			end

			EEAB_Data["Bindings"][bar][button] = key1
			EEAB_Data["Bindings"][key1] = {bar, button}
		else
			if EEAB_Data["Bindings"][bar] and EEAB_Data["Bindings"][bar][button] then
				EEAB_Data["Bindings"][EEAB_Data["Bindings"][bar][button]] = nil
				EEAB_Data["Bindings"][bar][button] = nil
			end
		end

		EEAB.Button_UpdateAll()

		EEAB.OptionsTab(ExtendedActionBar_Optionen_Tab2)

		if not EEAB_DefaultBindings then
			EEAB.GenerateDefaultBindings()
		end

		if EEAB_DefaultBindings[key1] then
			local key3, key4 = GetBindingKey(EEAB_DefaultBindings[key1])

			if key3 == key1 then
				SetBindingKey(EEAB_DefaultBindings[key1], key4, nil)

				EEAB_DefaultBindings = nil

			elseif key4 == key1 then
				SetBindingKey(EEAB_DefaultBindings[key1], key3, nil)

				EEAB_DefaultBindings = nil
			end
		end
	else
		SetBindingKey(action, key1, key2)
	end
end

function EEAB.DoSystemBinding(action, keystate)
	if action == "MOVEFORWARD" then
		if not keystate then
			MoveForwardStart()
		else
			MoveForwardStop()
		end
	elseif action == "MOVEBACKWARD" then
		if not keystate then
			MoveBackwardStart()
		else
			MoveBackwardStop()
		end
	elseif action == "STRAFELEFT" then
		if not keystate then
			StrafeLeftStart()
		else
			StrafeLeftStop()
		end
	elseif action == "STRAFERIGHT" then
		if not keystate then
			StrafeRightStart()
		else
			StrafeRightStop()
		end
	elseif action == "TURNLEFT" then
		if not keystate then
			TurnLeftStart()
		else
			TurnLeftStop()
		end
	elseif action == "TURNRIGHT" then
		if not keystate then
			TurnRightStart()
		else
			TurnRightStop()
		end
	elseif action == "PUSHTOTALK" then
		if not keystate then
			VoiceChat_PushToTalk_Start()
			PMF_MicIconButton:GetNormalTexture():SetFile("Interface\\PlayerFrame\\MicIcon_StartSpeak")
		else
			VoiceChat_PushToTalk_Stop()
			PMF_MicIconButton:GetNormalTexture():SetFile("Interface\\PlayerFrame\\MicIcon_Normal")
		end
	elseif string.find(action, "^ACTIONBAR%d*BUTTON%d*") then
		local search = string.gfind(action, "%d+")

		local slot = 20 * (search() - 1 ) + search()

		if not keystate then
			ActionButtonDown(slot)
		else
			ActionButtonUp(slot)
		end
	elseif string.find(action, "EXTRAACTIONBUTTON") then
		if keystate then
			keystate = "up" --?
		else
			keystate = "down"
		end

		if action == "EXTRAACTIONBUTTON1" then
			ExtraActionButton_Click(1, keystate)
		elseif action == "EXTRAACTIONBUTTON2" then
			ExtraActionButton_Click(2, keystate)
		elseif action == "EXTRAACTIONBUTTON3" then
			ExtraActionButton_Click(3, keystate)
		elseif action == "EXTRAACTIONBUTTON4" then
			ExtraActionButton_Click(4, keystate)
		elseif action == "EXTRAACTIONBUTTON5" then
			ExtraActionButton_Click(5, keystate)
		elseif action == "EXTRAACTIONBUTTON6" then
			ExtraActionButton_Click(6, keystate)
		elseif action == "EXTRAACTIONBUTTON7" then
			ExtraActionButton_Click(7, keystate)
		elseif action == "EXTRAACTIONBUTTON8" then
			ExtraActionButton_Click(8, keystate)
		elseif action == "EXTRAACTIONBUTTON9" then
			ExtraActionButton_Click(9, keystate)
		elseif action == "EXTRAACTIONBUTTON10" then
			ExtraActionButton_Click(10, keystate)
		end
	elseif not keystate then
		if action == "JUMP" then
			Jump()
		elseif action == "SITORSTAND" then
			SitOrStand()
		elseif action == "TOGGLEAUTORUN" then
			ToggleAutoRun()
		elseif action == "TOGGLERUN" then
			ToggleRun()
		elseif action == "TOGGLESHEATH" then
			ToggleSheath()
		elseif action == "FOLLOWTARGET" then
			FollowUnit("target")
		elseif action == "SCREENSHOT" then
			TakeScreenshot()
		elseif action == "TOGGLEBACKPACK" then
			ToggleBackpack()
		elseif action == "TOGGLECHARACTER0" then
			ToggleCharacter("EquipmentFrame")
		elseif action == "TOGGLECHARACTER1" then
			ToggleCharacter("PracticedFrame")
		elseif action == "TOGGLECHARACTER2" then
			ToggleCharacter("CCFrame")
		elseif action == "TOGGLECHARACTER3" then
			ToggleUIFrame(AchievementTitleFrame)
		elseif action == "TOGGLECHARACTER4" then
			ToggleCharacter("CEFrame")
		elseif action == "TOGGLECHARACTER5" then
			ToggleCharacter("CharacterClassListFrame")
		elseif action == "TOGGLESPELLBOOK" then
			ToggleUIFrame(UI_SkillBook)
		elseif action == "TOGGLEQUESTBOOK" then
			ToggleUIFrame(UI_QuestBook)
		elseif action == "TOGGLEGUILD" then
			ToggleUIFrame(GuildFrame)
		elseif action == "TOGGLEINSTANCERECORD" then
			ToggleUIFrame(InstanceRecordFrame)
		elseif action == "TOGGLEPARTERFRAME" then
			ToggleUIFrame(PartnerFrame)
		elseif action == "TOGGLEINFO" then
			ToggleSocialFrame("PersonalData")
		elseif action == "TOGGLESOCIALFRAME" then
			ToggleSocialFrame("Friend")
		elseif action == "TOGGLEBLACKLISTFRAME" then
			ToggleSocialFrame("BadFriend")
		elseif action == "TOGGLEHATEFRIEND" then
			ToggleSocialFrame("HateFriend")
		elseif action == "TOGGLESEARCHGROUP" then
			ToggleSocialFrame("SearchGroup")
		elseif action == "TOGGLECHARTBOOK" then
			ToggleUIFrame(CardBookFrame)
		elseif action == "TOGGLEBEAUTYSTUDIO" then
			if IsEnableMerchant() then
				ToggleUIFrame(BeautyStudioFrame)
			end
		elseif action == "TOGGLEITEMMALL" then
			if not IsBattleGroundZone() then
				ToggleUIFrame(ItemMallFrame)
			end
		elseif action == "TOGGLECRAFTFRAME" then
			ToggleUIFrame(CraftFrame)
		elseif action == "TOGGLEBULLETINBOARD" then
			ToggleUIFrame(BulletinBoard)
		elseif action == "TOGGLEWORLDMAP" then
			ToggleWorldMap()
		elseif action == "TOGGLEAREAMAP" then
			ToggleAreaMap()
		elseif action == "TOGGLENPCTRACK" then
			ToggleUIFrame(NpcTrackFrame)
		elseif action == "TOGGLETEACHING" then
			ToggleUIFrame(TeachingFrame)
		elseif action == "TOGGLEGAMEHELP" then
			ToggleUIFrame(GameIconFrame)
		elseif action == "TOGGLESORTSCOREBOARD" then
			ToggleUIFrame(SortScoreBoard)
		elseif action == "TOGGLEDRAWOUTRUNE" then
			ToggleUIFrame(DrawRuneFrame)
		elseif action == "TOGGLESKILLSUIT" then
			ToggleUIFrame(SkillSuitFrame)
		elseif action == "TOGGLETUTORIALEXPLAIN" then
			ToggleUIFrame(TutorialExplainFrame)
		elseif action == "TOGGLEFPSTEXT" then
			if FramerateText:IsVisible() then
				FramerateText:Hide()
			else
				FramerateText:Show()
			end
		elseif action == "TOGGLEUI" then
			if UIParent:IsVisible() then
				CloseAllWindows()
				UIParent:Hide()
			else
				UIParent:Show()
			end
		elseif action == "TOGGLEGAMEMENU" then
			if ToggleMsnWin() == false then
				ToggleGameMenu()
			end
		elseif action == "TOGGLENPCNAME" then
			ToggleUI_NPCNAME()
		elseif action == "TOGGLEHEALTH" then
			ToggleUI_ObjectBloodBar()
		elseif action == "TOGGLEPLATES" then
			ToggleUI_TITLE()
		elseif action == "OPENCHAT" then
			ChatEdit_OpenEditBox("")
		elseif action == "REPLY" then
			ChatEdit_ReplyTell()
		elseif action == "REPEAT" then
			ChatEdit_RepeatTell()
		elseif action == "CHATPAGEUP" then
			DEFAULT_CHAT_FRAME:ScrollUp()
		elseif action == "CHATPAGEDOWN" then
			DEFAULT_CHAT_FRAME:ScrollDown()
		elseif action == "CHATTOP" then
			DEFAULT_CHAT_FRAME:ScrollTop()
		elseif action == "CHATBOTTOM" then
			DEFAULT_CHAT_FRAME:ScrollBottom()
		elseif action == "CHAT_PARTY" then
			ChatEditMenu_Party()
		elseif action == "CHAT_GUILD" then
			ChatEditMenu_Guild()
		elseif action == "CHAT_SAY" then
			ChatEditMenu_Say()
		elseif action == "CHAT_YELL" then
			ChatEditMenu_Yell()
		elseif action == "CHAT_ZONE" then
			ChatEditMenu_Zone()
		elseif action == "MINIMAPZOOMIN" then
			MiniMapZoomIn()
		elseif action == "MINIMAPZOOMOUT" then
			MiniMapZoomOut()
		elseif action == "TARGETNEARESTENEMY" then
			TargetNearestEnemy()
		elseif action == "TARGETPREVIOUSENEMY" then
			TargetNearestEnemy(true)
		elseif action == "TARGETNEARESTFRIEND" then
			TargetNearestFriend()
		elseif action == "TARGETPREVIOUSFRIEND" then
			TargetNearestFriend(true)
		elseif action == "TARGETSELF" then
			TargetUnit("player")
		elseif action == "TARGETPARTY1" then
			TargetUnit("party1")
		elseif action == "TARGETPARTY2" then
			TargetUnit("party2")
		elseif action == "TARGETPARTY3" then
			TargetUnit("party3")
		elseif action == "TARGETPARTY4" then
			TargetUnit("party4")
		elseif action == "TARGETPARTY5" then
			TargetUnit("party5")
		elseif action == "TARGETPLAYERPET" then
			TargetUnit("playerpet")
		elseif action == "TARGETPARTY1PET" then
			TargetUnit("party1pet")
		elseif action == "TARGETPARTY2PET" then
			TargetUnit("party2pet")
		elseif action == "TARGETPARTY3PET" then
			TargetUnit("party3pet")
		elseif action == "TARGETPARTY4PET" then
			TargetUnit("party4pet")
		elseif action == "TARGETPARTY5PET" then
			TargetUnit("party5pet")
		elseif action == "ASSISTTARGET" then
			AssistUnit("target")
		elseif action == "RAIDTARGET1" then
			SetRaidTarget("target", 1)
		elseif action == "RAIDTARGET2" then
			SetRaidTarget("target", 2)
		elseif action == "RAIDTARGET3" then
			SetRaidTarget("target", 3)
		elseif action == "RAIDTARGET4" then
			SetRaidTarget("target", 4)
		elseif action == "RAIDTARGET5" then
			SetRaidTarget("target", 5)
		elseif action == "RAIDTARGET6" then
			SetRaidTarget("target", 6)
		elseif action == "RAIDTARGET7" then
			SetRaidTarget("target", 7)
		elseif action == "RAIDTARGET8" then
			SetRaidTarget("target", 8)
		elseif action == "PETACTIONBARBUTTON1" then
			UsePetAction(1)
		elseif action == "PETACTIONBARBUTTON2" then
			UsePetAction(2)
		elseif action == "PETACTIONBARBUTTON3" then
			UsePetAction(3)
		elseif action == "PETACTIONBARBUTTON4" then
			UsePetAction(4)
		elseif action == "PETACTIONBARBUTTON5" then
			UsePetAction(5)
		elseif action == "PETACTIONBARBUTTON6" then
			UsePetAction(6)
		elseif action == "PETACTIONBARBUTTON7" then
			UsePetAction(7)
		elseif action == "PETACTIONBARBUTTON8" then
			UsePetAction(8)
		elseif action == "PETACTIONBARBUTTON9" then
			UsePetAction(9)
		elseif action == "PETACTIONBARBUTTON10" then
			UsePetAction(10)
		end
	elseif not keystate then
		EEAB.Log("unknown Binding!")
		EEAB.Log(action)
	end
end
