local EEAB = _G.EEAB

function EEAB.Button_OnLoad(this)
	this:RegisterEvent("CURSOR_ITEM_UPDATE")
	this:RegisterEvent("PLAYER_BUFF_CHANGED")
	this:RegisterEvent("UNIT_BUFF_CHANGED")
	this:RegisterEvent("LOADING_END")
	this:RegisterEvent("LOADING_START")

	this:RegisterForClicks("LeftButton", "RightButton")
end

function EEAB.Button_OnEvent(this, event, arg1)
	if event == "CURSOR_ITEM_UPDATE" then
		EEAB.AddUpdate(this)

	elseif event == "PLAYER_BAG_CHANGED" or event == "PLAYER_INVENTORY_CHANGED" then
		EEAB.AddUpdate(this)

	elseif event == "SKILL_UPDATE_COOLDOWN" then
		EEAB.AddUpdate(this)

	elseif event == "MACROFRAME_UPDATE" then
		EEAB.CheckForMacro(this)

	elseif event == "LOADING_START" then
		if this:IsVisible() then
			this.loading = true
		else
			this.loading = false
		end

		this:Hide()

	elseif event == "LOADING_END" then
		EEAB.AddUpdate(this)

		this.loading = nil

	elseif event == "PLAYER_BUFF_CHANGED" or (event == "UNIT_BUFF_CHANGED" and arg1 == "player") then
		EEAB.AddUpdate(this)

	elseif event == "PLAYER_COOL_SUIT_CHANGED" then
		EEAB.CheckForSkin(this)
		EEAB.AddUpdate(this)

	elseif event == "SWAP_EQUIPMENT_SUCCESS" or event == "PLAYER_EQUIPMENT_UPDATE" then
		EEAB.CheckForEquip(this)
	end
end

function EEAB.Button_UpdateAll()
	for i = 1, EEAB.Bar_GetNum() do
		for j = 1, 20 do
			EEAB.AddUpdate(getglobal("ExtendedActionBar_"..i.."_Button"..tonumber(j)))
		end
	end
end

function EEAB.Button_Update(this)
	local texture, name, count, locked, wore, continued, invalid = EEAB.GetActionInfo(this)
	local info = {typ =  EEAB.Data(this, "typ")}
	local cooldown = 0

	local icon = getglobal(this:GetName().."_Icon")

	if texture then
		icon:Show()
		icon:SetTexture(texture)
	else
		icon:Hide()

		return
	end

	local buttonName = getglobal(this:GetName().."_Name")

	if not EEAB_Data.Options.names then
		buttonName:Show()
		buttonName:SetText(name)
	else
		buttonName:Hide()
	end

	cooldown = EEAB.Button_UpdateCooldown(this)

	local cover = getglobal(this:GetName().."_Invalid")

	if not EEAB_Data.Options.slim and (invalid or (cooldown ~= 0 and not cooldown == false)) then
		cover:Show()
	else
		cover:Hide()
	end

	if not EEAB_Data.Options.slim then
		EEAB.SetItemButtonCount(this, count)
		EEAB.SetItemButtonLuminance(this, count == 0, info.typ)
	end

	if wore then
		getglobal(this:GetName().."_Border"):SetTexture("Interface\\Buttons\\QuickSlot-Normal2")
	else
		getglobal(this:GetName().."_Border"):SetTexture("Interface\\Buttons\\QuickSlot-Normal")
	end

	if continued then
		if EEAB_Data.Options.bufftime then
			EEAB.BuffBarUpdate(getglobal(this:GetName().."_Buff"))
		end

		local that = getglobal(this:GetName().."_Continued")

		if not that:IsVisible() then
			that:Show()
		end
	else
		local that = getglobal(this:GetName().."_Continued")

		that:Hide()
		that:SetColor(1, 0.46, 0)

		that.duration = nil
		that.remaining = nil

		if EEAB_Data.Options.bufftime then
			getglobal(this:GetName().."_Buff"):Hide()
		end
	end

	EEAB.Button_UpdateBorder(this)
	EEAB.Button_UpdateHotkeys(this)

	if not this.registertyp or this.registertyp ~= info.typ then
		EEAB.Unregister(this)
		EEAB.Register(this)

		this.registertyp = info.typ
	end
end

function EEAB.BuffBarUpdate(this, elapsedTime)
	if EEAB_Data.Options.bufftime then
		if elapsedTime then
			if not this.counting then
				this.counting = 0
			else
				this.counting = this.counting + elapsedTime

				if this.counting >= 0.1 then
					elapsedTime = this.counting

					this.counting = 0
				else
					return
				end
			end
		end

		if not elapsedTime then
			EEAB.SaveBuffTimeOnFrame(this)

		elseif this.remaining then
			this.remaining = this.remaining - elapsedTime

			local dura, remain = this.duration, this.remaining
			local cdura = dura

			if this.remaining <= 15 then
				dura = 15
			end

			local red, green, blue = EEAB.TrafficColor(remain, cdura)

			this:SetBarColor(red, green, blue, 0.5)
			this:SetValue(remain / dura * 100)

			local text = ""

			if remain >= 86400 then
				text = EEAB.floor(remain / 86400, 1)..TEXT("UNIT_DAY")
			elseif remain >= 3600 then
				text = EEAB.floor(remain / 3600, 1)..TEXT("UNIT_HOUR")
			elseif remain >= 60 then
				text = EEAB.floor(remain / 60, 1)..TEXT("UNIT_MINUTE")
			elseif remain >= 1 then
				text = EEAB.floor(remain, 1)
			end

			getglobal(this:GetName().."_Time"):SetText(text)
		else
			this:Hide()
		end
	end
end

function EEAB.BuffBarLoad(this)
	this:RegisterEvent("UNIT_BUFF_CHANGED")
end

function EEAB.BuffBarHide(this)
	this.duration = nil
	this.remaining = nil

	this:SetValue(100)
	this:SetBarColor(0, 1, 0, 0.5)

	getglobal(this:GetName().."_Time"):SetText("")
end

function EEAB.BuffBarEvent(this, event, arg1)
	if event == "UNIT_BUFF_CHANGED" and arg1 == "player" then
		EEAB.BuffBarUpdate(this)
	end
end

function EEAB.Button_UpdateBorder(this)
	local visible

	if EEAB.HasAction(this) or not EEAB_Data.Options.hideempty or CursorHasItem() then
		visible = true
	end

	if this.visible and visible then
		this:Show()
	else
		this:Hide()
	end
end

function EEAB.Button_UpdateCooldown(this)
	local duration, remaining = EEAB.GetActionCooldown(this)

	if duration and not getglobal(this:GetName().."_Cooldown").starting then
		EEAB.CooldownFrame_SetTime(getglobal(this:GetName().."_Cooldown"), duration, remaining)
	end

	return remaining
end

function EEAB.Button_UpdateHotkeys(this)
	local hotkey = getglobal(this:GetName().."_Hotkey")
	local key = EEAB.GetBindingKey("EXTENDEDACTIONBAR"..EEAB.null(this:GetParent():GetID()).."BUTTON"..EEAB.null(this:GetID()))

	if key and EEAB_Data.Options.hotkeys then
		hotkey:SetText(key)
	else
		hotkey:SetText("")
	end
end

function EEAB.Button_OnHideCooldown(this)
	local name = EEAB.Data(this:GetParent(), "name")

	if name and EEAB.LastCooldown then
		EEAB.LastCooldown[name] = nil
	end

	if not EEAB_Data.Options.slim then
		EEAB.Button_Update(this:GetParent())
	end
end

function EEAB.ScaleButton(button, scale)
	if button:GetSize() ~= scale then
		button:SetSize(scale, scale)

		getglobal(button:GetName().."_Border"	):SetSize(scale * 1.8,	 scale * 1.8)
		getglobal(button:GetName().."_Icon"		):SetSize(scale / 1.125, scale / 1.125)
		getglobal(button:GetName().."_Invalid"	):SetSize(scale / 1.125, scale / 1.125)
		getglobal(button:GetName().."_Continued"):SetSize(scale / 1.125, scale / 1.125)
		getglobal(button:GetName().."_Cooldown"	):SetSize(scale / 1.125, scale / 1.125)
		getglobal(button:GetName().."_Hotkey"	):SetSize(scale / 1.125, scale / 1.125 / 2)
		getglobal(button:GetName().."_Name"		):SetSize(scale / 1.125, scale / 1.125 / 2)
	end
end

function EEAB.ContinuedUpdate(this, elapsedTime)
	if not EEAB_Data.Options.slim and this:IsVisible() then
		if not this.counting then
			this.counting = 0
		else
			this.counting = this.counting + elapsedTime

			if this.counting >= 0.1 then
				elapsedTime = this.counting

				this.counting = 0
			else
				return
			end
		end

		if this.remaining then
			this.remaining = this.remaining - elapsedTime

			local red, green, blue = EEAB.TrafficColor(this.remaining, this.duration)

			this:SetColor(red, green, blue)
		else
			EEAB.SaveBuffTimeOnFrame(this)
		end
	end
end
