local EEAB = _G.EEAB

function EEAB.DataManager_EditBox(this)
	local buttonID, value = this:GetID(), this:GetText()

	local bar = tonumber(ExtendedActionBar_DataManager_Bar:GetText()) or 0
	local button = tonumber(ExtendedActionBar_DataManager_Button:GetText()) or 0


	local valid = bar <= EEAB.Bar_GetNum() and button <= 20 and bar > 0 and button > 0

	if valid and (buttonID == 1 or buttonID == 2) then
		local hash = EEAB.null(bar)..EEAB.null(button)

		if not EEAB.DropHash or EEAB.DropHash ~= hash then
			EEAB.DropHash = hash

			local info = EEAB_Data[EEAB.GetClass()][bar][button]

			if info.typ then
				EEAB.DataManager_Set(info.typ, info.name, info.texture, info.count, info.ID, info.slot, info.indent)
			else
				EEAB.DataManager_Set("default")
			end
		end

	elseif not valid then
		EEAB.DataManager_Set()
	end
end

function EEAB.DataManager_Save()
	local info = {
		typ		= EEAB.DataType,
		name	= ExtendedActionBar_DataManager_Name:GetText(),
		texture	= ExtendedActionBar_DataManager_Texture:GetText(),
		count	= ExtendedActionBar_DataManager_Count:GetText(),
		ID		= ExtendedActionBar_DataManager_Id:GetText(),
		slot	= ExtendedActionBar_DataManager_Slot:GetText(),
		indent	= ExtendedActionBar_DataManager_Indent:GetText(),
	}

	local done = EEAB.CheckValidSave(info)

	if done then
		EEAB.io(91)

	elseif done ~= nil then
		EEAB.io(90)
	else
		EEAB.io(92)
	end

	ExtendedActionBar_DataManager_Bar:SetText("")
	ExtendedActionBar_DataManager_Button:SetText("")

	EEAB.DropHash = nil

	EEAB.Bar_UpdateAll()
end

function EEAB.CheckValidSave(info)
	local texture, bool = info.texture
	local bar = tonumber(ExtendedActionBar_DataManager_Bar:GetText()) or 0
	local button = tonumber(ExtendedActionBar_DataManager_Button:GetText()) or 0

	local valid = bar <= EEAB.Bar_GetNum() and button <= 20 and bar > 0 and button > 0

	if info.name == "" then
		info.name = nil
	end
	if info.texture == "" then
		info.texture = nil
	end
	if info.count == "" then
		info.count = nil
	end
	if info.ID == "" then
		info.ID = nil
	end
	if info.slot == "" then
		info.slot = nil
	end
	if info.indent == "" then
		info.indent = nil
	end

	if valid then
		if info.typ == "bag" then
			if info.name then
				for i = 1, 360 do
					info.ID, info.texture, name, info.count = GetBagItemInfo(i)

					if name and name == info.name then
						info.slot = i

						bool = true

						break
					end

					if bool then
						break
					end
				end
			end

		elseif info.typ == "itemshop" then
			if info.name then
				for i = 1, 50 do
					info.slot = i
					info.ID = i
					info.texture, name, info.count = GetGoodsItemInfo(i)

					if name == info.name then
						bool = true

						break
					end
				end
			end

		elseif info.typ == "skill" then
			if info.slot and info.ID then
				info.name, _, info.texture = GetSkillDetail(info.slot, info.ID)

				if info.name then
					bool = true
				end

			elseif info.name then
				for i = 1, 4 do
					for j = 1, 40 do
						local name, _, texture = GetSkillDetail(i, j)

						if name == info.name then
							info.slot, info.ID, info.texture = i, j, texture
							bool = true

							break
						end
					end

					if bool then
						break
					end
				end
			end

		elseif info.typ == "emote" then
			if info.ID then
				_, info.name, _, info.texture = GetEmoteInfo(info.ID)

				if info.name then
					bool = true
				end
			end

		elseif info.typ == "SuitSkill" then
			if info.name then
				for i = 0, 11 do
					for j = 0, 10 do
						local a, b, c = GetSuitSkill_List(i, j)

						if a == info.name then
							info.slot, info.ID, info.count = i, j, c
							bool = true

							break
						end
					end

					if bool then
						break
					end
				end

			elseif info.slot and info.ID then
				info.name, info.texture, info.count = GetSuitSkill_List(info.slot, info.ID)

				if info.name then
					bool = true
				end
			end

		elseif info.typ == "macro" then
			if info.ID then
				info.count, info.name = GetMacroInfo(info.ID)

				if info.name then
					info.texture = GetMacroIconInfo(info.count)
					bool = true
				end
			end

		elseif info.typ == "HappyPartner" then
			if info.slot and info.ID then
				info.texture, _, info.name = PartnerFrame_GetPartnerInfo(info.slot, info.ID)

				if info.name then
					bool = true
				end
			end

		elseif info.typ == "teleport" then
			if info.slot then
				_, info.name, _, _, _, _, _, info.texture = TB_GetTeleportInfo(info.slot)

				if info.name then
					bool = true
				end
			end

		elseif info.typ == "CoolSuit" then
			if info.slot then
				info.name = string.gsub(TEXT("SYS_COOLSUIT_PAGE_TIPS"), "<number>", tonumber(info.slot))

				_, info.texture = CoolSuit_GetItemInfo(info.slot, 3)

				if not info.texture then
					for i = 1, 9 do
						_, info.texture = CoolSuit_GetItemInfo(info.slot, i)

						if info.texture then
							break
						end
					end
				end

				if not info.texture then
					info.texture = CoolSuit_GetPageTexture(info.slot)
				end

				if info.name then
					bool = true
				end
			end

		elseif info.typ == "equip" then
			if info.ID then
				info.texture = GetInventoryItemTexture("player", info.ID)

				local frames = {"Ring0Slot", "Ring1Slot", "Earring0Slot", "Earring1Slot", "NecklaceSlot", "Talisman0Slot", "Talisman1Slot", "Talisman2Slot", "HeadSlot", "ChestSlot", "WaistSlot", "LegsSlot", "MainHandSlot", "RangedSlot", "ShoulderSlot", "HandsSlot", "FeetSlot", "BackSlot", "SecondaryHandSlot", "AdornmentSlot"}

				for _, name in pairs(frames) do
					if GetInventorySlotInfo(name) == info.ID then
						info.name = name

						break
					end
				end

				if info.name then
					bool = true
				end
			end

		elseif info.typ == "CoolClothBag" then
			if info.ID then
				info.name, info.texture = CoolClothBag_GetItem(info.ID)
				info.name = TEXT("Sys"..info.name.."_name")

				if info.name then
					bool = true
				end
			end

		elseif info.typ == "title" then
			if info.name then
				for i = 1 , GetTitleCount() do
					name, slot, _, texture = GetTitleInfoByIndex(i - 1)

					if string.lower(info.name) == string.lower(name) or info.name == slot then
						info.name = name
						info.slot = slot
						info.texture = texture

						bool = true

						break
					end
				end
			end

		elseif info.typ == "pet" then
			if info.slot then
				info.name = GetPetItemName(info.slot)
				info.texture = "interface/icons/shop_goods/pet_egg_06.dds"

				if info.name then
					bool = true
				end
			end

		elseif info.typ == "equipslot" then
			info.slot = tonumber(info.slot)

			if info.slot and info.slot >= 0 and info.slot <= 2 then
				info.name = "("..(info.slot + 1)..") "..TEXT("SWAP_EQUIPMENT_TOOLTIP")
				info.texture = "interface/CharacterFrame/EquipSwapButton-Normal"
				bool = true
			end
		end

		if bool and info.name then
			if texture and texture ~= "" then
				info.texture = texture
			end

			EEAB_Data[EEAB.GetClass()][bar][button] = info
			EEAB.DataManager_Set()

			return true
		else
			return false
		end
	else
		return false
	end
end

function EEAB.DataManager_Tooltip(this)
	local id = 109

	if this:GetID() == 1 then
		id = 94

	elseif this:GetID() == 2 then
		id = 95

	elseif EEAB.DataType == "bag" and this:GetID() == 3 then
		id = 93

	elseif EEAB.DataType == "itemshop" and this:GetID() == 3 then
		id = 93

	elseif this:GetID() == 4 then
		id = 128

	elseif EEAB.DataType == "skill" and (this:GetID() == 3 or this:GetID() == 6 or this:GetID() == 7) then
		if this:GetID() == 7 then
			id = 96
		elseif this:GetID() == 6 then
			id = 97
		else
			id = 101
		end

	elseif EEAB.DataType == "emote" and this:GetID() == 6 then
		id = 98

	elseif EEAB.DataType == "SuitSkill" and (this:GetID() == 3 or this:GetID() == 6 or this:GetID() == 7) then

		if this:GetID() == 7 then
			id = 99
		elseif this:GetID() == 6 then
			id = 100
		else
			id = 101
		end

	elseif EEAB.DataType == "macro" and this:GetID() == 6 then
		id = 102

	elseif EEAB.DataType == "HappyPartner" and (this:GetID() == 6 or this:GetID() == 7) then
		if this:GetID() == 7 then
			id = 103
		else
			id = 104
		end

	elseif EEAB.DataType == "teleport" and this:GetID() == 7 then
		id = 105

	elseif EEAB.DataType == "CoolSuit" and this:GetID() == 7 then
		id = 106

	elseif EEAB.DataType == "equip" and this:GetID() == 6 then
		id = 107

	elseif EEAB.DataType == "CoolClothBag" and this:GetID() == 6 then
		id = 108

	elseif EEAB.DataType == "title" and this:GetID() == 3 then
		id = 112

	elseif EEAB.DataType == "pet" and this:GetID() == 7 then
		id = 110

	elseif EEAB.DataType == "equipslot" and this:GetID() == 7 then
		id = 111
	end

	EEAB.GameTooltip(this, "ANCHOR_BOTTOMRIGHT", 4, 0, id)
end

function EEAB.DataManager_SetDropDown(this)
	local category = this.arg1

	if category ~= 1 then
		local typ, name, texture, count, ID, slot, indent = category, "", "", 0, 1, 1, 1

		if category == "bag" then
			texture, count, ID, slot, indent = nil, nil, nil, nil, nil

		elseif category == "itemshop" then
			texture, count, ID, slot, indent = nil, nil, nil, nil, nil

		elseif category == "skill" then
			name, texture, count, indent = nil, nil, nil, nil

		elseif category == "emote" then
			name, texture, count, slot, indent = nil, nil, nil, nil, nil

		elseif category == "SuitSkill" then
			texture, count, indent = nil, nil, nil

		elseif category == "macro" then
			name, texture, count, slot, indent = nil, nil, nil, nil, nil

		elseif category == "HappyPartner" then
			name, texture, count, indent = nil, nil, nil, nil

		elseif category == "teleport" then
			name, texture, count, ID, indent = nil, nil, nil, nil, nil

		elseif category == "CoolSuit" then
			name, texture, count, ID, indent = nil, nil, nil, nil, nil

		elseif category == "equip" then
			name, texture, count, slot, indent = nil, nil, nil, nil, nil

		elseif category == "CoolClothBag" then
			name, texture, count, slot, indent = nil, nil, nil, nil, nil

		elseif category == "title" then
			texture, count, ID, slot, indent = nil, nil, nil, nil, nil

		elseif category == "pet" then
			name, texture, count, ID, indent = nil, nil, nil, nil, nil

		elseif category == "equipslot" then
			name, texture, count, ID, indent = nil, nil, nil, nil, nil
		end

		EEAB.DataManager_Set(typ, name, texture, count, ID, slot, indent)
	else
		EEAB.DataManager_Set("default")
	end
end

function EEAB.DataManager_Set(typ, name, texture, count, ID, slot, indent)
	if typ then
		ExtendedActionBar_DataManager_Type:Show()
		UIDropDownMenu_SetSelectedID(ExtendedActionBar_DataManager_Type, EEAB.typ[typ])

		EEAB.DataType = typ
	else
		EEAB.DataType = nil
		ExtendedActionBar_DataManager_Type:Hide()
	end

	if name then
		ExtendedActionBar_DataManager_Name:Show()
		ExtendedActionBar_DataManager_Name:SetText(name)
	else
		ExtendedActionBar_DataManager_Name:Hide()
	end

	if texture then
		ExtendedActionBar_DataManager_Texture:Show()
		ExtendedActionBar_DataManager_Texture:SetText(texture)
	else
		ExtendedActionBar_DataManager_Texture:Hide()
	end

	if count then
		ExtendedActionBar_DataManager_Count:Show()
		ExtendedActionBar_DataManager_Count:SetText(count)
	else
		ExtendedActionBar_DataManager_Count:Hide()
	end

	if ID then
		ExtendedActionBar_DataManager_Id:Show()
		ExtendedActionBar_DataManager_Id:SetText(ID)
	else
		ExtendedActionBar_DataManager_Id:Hide()
	end

	if slot then
		ExtendedActionBar_DataManager_Slot:Show()
		ExtendedActionBar_DataManager_Slot:SetText(slot)
	else
		ExtendedActionBar_DataManager_Slot:Hide()
	end

	if indent then
		ExtendedActionBar_DataManager_Indent:Show()
		ExtendedActionBar_DataManager_Indent:SetText(indent)
	else
		ExtendedActionBar_DataManager_Indent:Hide()
	end
end
